package XLS_hw;

use strict;
use Exporter;
use LCSS;

our @ISA    = qw(Exporter);
our @EXPORT = qw( hw_configuration);

my %hw_info = ();

sub hw_query {
    $hw_info{lshw}    = join( "", `lshw 2>/dev/null` );
    $hw_info{udevadm} = join( "", `udevadm info --export-db 2>/dev/null` );
    $hw_info{meminfo} = join( "", `cat /proc/meminfo 2>/dev/null` );
    @{ $hw_info{lspci} } = `lspci 2>/dev/null`;
    $hw_info{bios_version} =
      `cat /sys/devices/virtual/dmi/id/bios_version 2>/dev/null`;
    $hw_info{sys_vendor} =
      `cat /sys/devices/virtual/dmi/id/sys_vendor 2>/dev/null`;
    $hw_info{product_name} =
      `cat /sys/devices/virtual/dmi/id/product_name 2>/dev/null`;

}

sub hw_configuration {
    my $conf = shift;
    hw_query();

    # Platform name
    if ( $hw_info{product_name} and $hw_info{sys_vendor} ) {
        $hw_info{$_} =~ s/\n$//g for ( 'sys_vendor', 'product_name' );
        $conf->{platform} = "$hw_info{sys_vendor} $hw_info{product_name}";
    }
    else {
        $conf->{platform} = 'NA';
    }

    # BIOS version
    if ( $hw_info{bios_version} ) {
        chomp( $conf->{bios} = $hw_info{bios_version} );
    }
    else {
        $conf->{bios} = 'NA';
    }

    # Bluetooth
    if ( $hw_info{udevadm}
        and ( $hw_info{udevadm} =~ /ID_MODEL=(.*?bluetooth.*)\n/i ) )
    {
        $conf->{bluetooth} = $1;
        $conf->{bluetooth} =~ s/_/ /g;
    }
    else {
        $conf->{bluetooth} = 'NA';
    }

    # Chipset (guessed from lspci output with LCSS)
    if ( $hw_info{lspci} ) {
        $conf->{chipset} = lcss( @{ $hw_info{lspci} } );
        $conf->{chipset} =~ s/.*:\s+//;
    }
    else {
        $conf->{chipset} = 'NA';
    }

    # MemTotal
    if ( $hw_info{meminfo}
        and ( $hw_info{meminfo} =~ /memtotal:\s*(\d+)\s*/i ) )
    {
        $conf->{memory} = $1;
        $conf->{memory} = ( int( $conf->{memory} / 1024 ) ) . " MiB";
    }
    else {
        $conf->{memory} = 'NA';
    }

    # VRAM
    $conf->{vram} = 'NA';
    my $r = `cat /var/log/Xorg.0.log`;
    if ( $r =~ /video\s*ram[:=]\s*(\d+)/i ) {
        $conf->{vram} = ( $1 / 1024 ) . " MiB";
    }
    elsif ( $r = `nvidia-settings -t -query Videoram 2>/dev/null` / 1024 ) {
        $conf->{vram} = "$r MiB";
    }

    # Info extracted from lshw
    my %devices = (
        audio => 'Audio device',
        vga1  => 'VGA compatible controller',
        vga2  => undef,
        wlan  => 'Wireless interface',
        nic   => 'Ethernet interface',
    );
    if ( $hw_info{lshw} ) {
        foreach my $d ( keys %devices ) {
            next unless $devices{$d};

            my @matches = ();
            while ( $hw_info{lshw} =~
/: $devices{$d}.*?(?:(?:product:\s+(.*?)\n.*?vendor:\s+(.*?)\n.*?version:\s+(.*?))|(?:(?:speed|wireless)=(.*?)))\n/gims
              )
            {
                if ( $1 and $2 and $3 ) {
                    push( @matches, "$2 $1 (rev $3)" );
                }
                else {
                    push( @matches, "$4" );
                }

            }
            chomp( $conf->{$d} = join( "\n", @matches ) );

            # Manage multiple vga controllers
            if ( ( $d eq 'vga1' ) and $#matches ) {
                $conf->{vga1} = shift @matches;
                chomp( $conf->{vga2} = join( "\n", @matches ) );
            }
        }
    }
    else {
        $conf->{$_} = 'NA' for keys %devices;
    }
    $conf->{$_} = $conf->{$_} eq '' ? 'NA' : $conf->{$_}
      for keys %{$conf};
}

1;

__END__


