"""
Backup plugin used to to copy the whole contents
of $CHECKBOX_DATA under $CHECKBOX_DATA/backup
before test results are submitted to make sure
that data isn't lost in case of an error
during test results submission
"""
import os
import shutil
import time
import tarfile

from checkbox.plugin import Plugin

class Backup(Plugin):
    """
    Backup all files under $CHECKBOX_DATA directory
    """
    def register(self, manager):
        super(Backup, self).register(manager)

        # Callback priority (-1) is set to execute
        # the backup before the callbacks
        # that are in charge of test results submission
        self._manager.reactor.call_on("report-exchange",
                                      self.report_exchange, -1)


    def report_exchange(self, interface):
        """
        Backup files
        """
        home_dir = os.getenv('HOME')
        data_dir = os.getenv('CHECKBOX_DATA')
        timestamp = time.strftime('%Y%m%dT%H%M%SZ', time.gmtime())
        filename = 'checkbox-'+timestamp+'.tar.gz'
        backup_dir = 'checkbox-backups'
        backup_path = os.path.join(home_dir, backup_dir)
        
        # Create the backup dir if it doesn't exist
        if not os.path.exists(backup_path):
            os.mkdir(backup_path)
        
        tarball = tarfile.open(os.path.join(backup_path, filename), 'w:gz')

        # Get the file list from CHECKBOX_DATA and add them to the tarball
        # arcname strips the leading path of the files in the archive so when
        # you untar you don't end up recreating the entire
        # /$HOME/$CHECKBOX_DATA hieracrhy.
        for file in os.listdir(data_dir):
            tarball.add(os.path.join(data_dir,file),arcname=file)

        tarball.close()

factory = Backup
