from checkbox.plugin import Plugin
from checkbox.properties import Float, Int, List, Map, Message, String


attachment_schema = Map({
    "name": String(),
    "content": String(),
    "project": String(required=False),
    "job": String(required=False)})

device_schema = Map({
    "path": String(),
    "bus": String(),
    "driver": String(required=False),
    "product_id": Int(required=False),
    "vendor_id": Int(required=False),
    "subproduct_id": Int(required=False),
    "subvendor_id": Int(required=False),
    "product": String(required=False),
    "vendor": String(required=False),
    "category": String(required=False)})

disk_schema = Map({
    "distributor": String(),
    "release": String(),
    "codename": String(),
    "official": String(),
    "architecture": String(),
    "date": String()})

distribution_schema = Map({
    "distributor_id": String(),
    "description": String(),
    "release": String(),
    "codename": String()})

memory_schema = Map({
    "total": Int(),
    "swap": Int()})

package_schema = Map({
    "name": String(),
    "version": String()})

processor_schema = Map({
    "platform": String(),
    "count": Int(),
    "type": String(),
    "model": String(),
    "model_number": String(),
    "model_version": String(),
    "model_revision": String(),
    "cache": Int(),
    "bogomips": Int(),
    "speed": Int(),
    "other": String()})

job_schema = Map({
    "type": String(),
    "name": String(),
    "project": String(required=False),
    "description": String(),
    "annotation": String(required=False),
    "command": String(required=False),
    "environ": List(String(), required=False),
    "depends": List(String(), required=False),
    "requires": String(required=False),
    "timeout": Int(required=False),
    "user": String(required=False)})

result_schema = Map({
    "type": String(),
    "name": String(),
    "project": String(required=False),
    "status": String(),
    "value": String(),
    "duration": Float(required=False),
    "devices": List(device_schema, required=False),
    "packages": List(package_schema, required=False)})

ERROR = Message("error", {
        "message": String()})

CREATE_SUBMISSION = Message("create-submission", {
        "tags": String(required=False)})

REPORT_SUBMISSION = Message("report-submission", {
        "secure_id": String()})

SET_ARCHITECTURE = Message("set-architecture", {
        "architecture": String()})

SET_DEVICES = Message("set-devices", {
        "devices": List(device_schema)})

SET_DISK = Message("set-disk", {
        "disk": disk_schema})

SET_DISTRIBUTION = Message("set-distribution", {
        "distribution": distribution_schema})

SET_MEMORY = Message("set-memory", {
        "memory": memory_schema})

SET_PACKAGES = Message("set-packages", {
        "packages": List(package_schema)})

SET_PROCESSOR = Message("set-processor", {
        "processor": processor_schema})

SET_TAGS = Message("set-tags", {
        "tags": String()})

ADD_ATTACHMENTS = Message("add-attachments", {
        "attachments": List(attachment_schema)})

ADD_JOBS = Message("add-jobs", {
        "jobs": List(job_schema)})

ADD_RESULTS = Message("add-results", {
        "results": List(result_schema)})


class CertifySchemas(Plugin):

    def register(self, manager):
        super(CertifySchemas, self).register(manager)

        self._manager.reactor.call_on("gather", self.gather, -1000)

    def gather(self):
        for schema in [ERROR, CREATE_SUBMISSION, REPORT_SUBMISSION,
                SET_ARCHITECTURE, SET_DISK, SET_DISTRIBUTION, SET_MEMORY,
                SET_DEVICES, SET_PACKAGES, SET_PROCESSOR, SET_TAGS,
                ADD_ATTACHMENTS, ADD_JOBS, ADD_RESULTS]:
            self._manager.reactor.fire("gather-schema", schema)


factory = CertifySchemas
