from checkbox.contrib.persist import Persist, MemoryBackend

from checkbox.plugin import Plugin
from checkbox.properties import String


class SubmissionInfo(Plugin):

    # Secure ID of the submission
    secure_id = String(required=False)

    # Tags for the submission
    tags = String(required=False)

    @property
    def persist(self):
        if self._persist is None:
            self._persist = Persist(backend=MemoryBackend())

        persist = self._persist.root_at("submission_info")
        if not persist.has("secure-id"):
            persist.set("secure-id", self.secure_id)

        return persist

    def register(self, manager):
        super(SubmissionInfo, self).register(manager)

        self._persist = None

        for (rt, rh) in [
             ("begin-persist", self.begin_persist),
             ("transport-send", self.transport_send)]:
            self._manager.reactor.call_on(rt, rh)

        # Report the submission id as early as possible
        self._manager.reactor.call_on("gather",
            self.gather_submission, -1000)

        # But report the tags as late as possible
        self._manager.reactor.call_on("gather",
            self.gather_tags, 1000)

    def begin_persist(self, persist):
        self._persist = persist

    def gather_submission(self):
        # Only use the configuration secure_id once
        secure_id = self.persist.get("secure-id")
        if secure_id:
            message = {"secure_id": secure_id}
            self._manager.reactor.fire("report-submission", message)

    def gather_tags(self):
        if self.tags:
            message = {"tags": self.tags}
            self._manager.reactor.fire("report-tags", [message])

    def transport_send(self, request):
        self.persist.set("secure-id", None)


factory = SubmissionInfo
